@@ -80,7 +80,7 @@ const char copyright[] = "Copyright (c) 2006-2012 Roy Marples";
 #define RELEASE_DELAY_S		0
 #define RELEASE_DELAY_NS	10000000
 
-int options = 0;
+unsigned long long options = 0;
 int pidfd = -1;
 struct interface *ifaces = NULL;
 int ifac = 0;
@@ -146,15 +146,24 @@ read_pid(void)
 static void
 usage(void)
 {
-	printf("usage: "PACKAGE" [-dgknpqwxyADEGHJKLOTV] [-c script] [-f file]"
-	    " [-e var=val]\n"
-	    "              [-h hostname] [-i classID ] [-l leasetime]"
-	    " [-m metric] [-o option]\n"
-	    "              [-r ipaddr] [-s ipaddr] [-t timeout]"
-	    " [-u userclass]\n"
-	    "              [-F none|ptr|both] [-I clientID] [-C hookscript]"
-	    " [-Q option]\n"
-	    "              [-X ipaddr] <interface>\n");
+
+printf("usage: "PACKAGE"\t[-ABbDdEGgHJKkLnpqTVw]\n"
+	"\t\t[-C, --nohook hook] [-c, --script script]\n"
+	"\t\t[-e, --env value] [-F, --fqdn FQDN] [-f, --config file]\n"
+	"\t\t[-h, --hostname hostname] [-I, --clientid clientid]\n"
+	"\t\t[-i, --vendorclassid vendorclassid] [-l, --leasetime seconds]\n"
+	"\t\t[-m, --metric metric] [-O, --nooption option]\n"
+	"\t\t[-o, --option option] [-Q, --require option]\n"
+	"\t\t[-r, --request address] [-S, --static value]\n"
+	"\t\t[-s, --inform address[/cidr]] [-t, --timeout seconds]\n"
+	"\t\t[-u, --userclass class] [-v, --vendor code, value]\n"
+	"\t\t[-W, --whitelist address[/cidr]] [-y, --reboot seconds]\n"
+	"\t\t[-X, --blacklist address[/cidr]] [-Z, --denyinterfaces pattern]\n"
+	"\t\t[-z, --allowinterfaces pattern] [interface] [...]\n"
+	"       "PACKAGE"\t-k, --release [interface]\n"
+	"       "PACKAGE"\t-U, --dumplease interface\n"
+	"       "PACKAGE"\t-v, --version\n"
+	"       "PACKAGE"\t-x, --exit [interface]\n");
 }
 
 static void
@@ -828,7 +837,7 @@ configure_interface1(struct interface *iface)
 	if (ifo->options & DHCPCD_CLIENTID)
 		syslog(LOG_DEBUG, "%s: using ClientID %s", iface->name,
 		    hwaddr_ntoa(iface->clientid + 1, *iface->clientid));
-	else
+	else if (iface->hwlen)
 		syslog(LOG_DEBUG, "%s: using hwaddr %s", iface->name,
 		    hwaddr_ntoa(iface->hwaddr, iface->hwlen));
 }
@@ -1162,8 +1171,12 @@ start_interface(void *arg)
 	free(iface->state->offer);
 	iface->state->offer = NULL;
 
-	if (options & DHCPCD_IPV6RS && ifo->options & DHCPCD_IPV6RS)
-		ipv6rs_start(iface);
+	if (options & DHCPCD_IPV6RS && ifo->options & DHCPCD_IPV6RS) {
+		if (check_ipv6(iface->name) == 1)
+			ipv6rs_start(iface);
+		else
+			ifo->options &= ~DHCPCD_IPV6RS;
+	}
 
 	if (iface->state->arping_index < ifo->arping_len) {
 		start_arping(iface);
@@ -1520,7 +1533,7 @@ handle_signal(_unused void *arg)
 			ifo->options |= DHCPCD_DAEMONISED;
 		options = ifo->options;
 		free_options(ifo);
-		reconf_reboot(1, 0, NULL, 0);
+		reconf_reboot(1, ifc, ifv, 0);
 		return;
 	case SIGHUP:
 		syslog(LOG_INFO, "received SIGHUP, releasing");
@@ -1993,7 +2006,7 @@ main(int argc, char **argv)
 	}
 #endif
 
-	if (options & DHCPCD_IPV6RS && !check_ipv6())
+	if (options & DHCPCD_IPV6RS && !check_ipv6(NULL))
 		options &= ~DHCPCD_IPV6RS;
 	if (options & DHCPCD_IPV6RS) {
 		ipv6rsfd = ipv6rs_open();
